import ark_algebra_py
from ark_algebra_py import ark_algebra_py

__doc__ = ark_algebra_py.ark_algebra_py.__doc__
if hasattr(ark_algebra_py.ark_algebra_py, "__all__"):
    __all__ = ark_algebra_py.ark_algebra_py.__all__